#include <sys/types.h>
#include <unistd.h>
#include "fork.h"
#include "wait.h"
#include "sig.h"
#include "exitcodes.h"
#include "strerr.h"

extern int flaglog;
extern pid_t pid;
extern int alarm_flag;
extern char *auto_scancmd[];

int do_scan()
{
  int avstat = -1;

  /* Defer alarms */
  sig_alarmblock();

  /* Launch antivir */
  switch(pid = vfork()) {
    case -1: /* Can't launch; temporary failure */
        if (flaglog)
          strerr_warn1("qscanq-stdin: fatal: vfork failed: ",&strerr_sys);
	return EX_TMPERR;
    case 0:
	close(1); /* Don't let it fiddle with envelope */
	if (!flaglog) close(2); /* Don't let it squeal, except in debug mode */
	execve(*auto_scancmd, auto_scancmd, 0);
        if (flaglog)
          strerr_warn1("qscanq-stdin: fatal: failed to execve scanner: ",&strerr_sys);
	_exit(QQ_XBUGS); /* hopefully never reached */;
  }

  /* Allow alarms */
  sig_alarmunblock();

  /* Catch the exit status */
  if (wait_pid(&avstat, pid) == -1) {
    if (flaglog)
      strerr_warn1("qscanq-stdin: waitpid failed: ",&strerr_sys);
    return EX_TMPERR; pid=0;
  }
  if (wait_crashed(avstat)) {
    if (flaglog)
      strerr_warn1("qscanq-stdin: virus scanner crashed: ",&strerr_sys);
    return EX_TMPERR;
  }
  return wait_exitcode(avstat);
}
