#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include "strerr.h"
#include "error.h"
#include "buffer.h"
#include "open.h"

extern const char *FATAL;

char *fn;
char *r;
char *d; /* must start and end with slash */
char buf[256];
buffer obuf;

void attrs(const char*fn, mode_t mode, uid_t uid, gid_t gid)
{
  if (chown(fn,uid,gid) == -1)
    strerr_die4sys(111,FATAL,"unable to chown ",fn,": ");
  if (chmod(fn,mode) == -1)
    strerr_die4sys(111,FATAL,"unable to chmod ",fn,": ");
}

void makedir(const char *dir,mode_t mode,uid_t uid, gid_t gid)
{
  if (mkdir(dir, 0700) == -1 && errno != error_exist)
    strerr_die4sys(111,FATAL,"unable to mkdir ",dir,": ");
  attrs(dir,mode,uid,gid);
}

int file_exists(void)
{
  struct stat st;
  if (stat(fn,&st) == -1) {
    if (errno == error_noent) return 0;
    else strerr_die6sys(111, FATAL,"unable to stat ",d,"/",fn,": ");
  }
  return 1;
}

void start(void)
{
  int fdout = open_trunc(fn);
  if (fdout == -1)
    strerr_die6sys(111,FATAL,"unable to write ",r,d,fn,": ");
  buffer_init(&obuf,buffer_unixwrite,fdout,buf,sizeof(buf));
}

void outs(const char *s)
{
  if (buffer_puts(&obuf,s) == -1)
    strerr_die6sys(111,FATAL,"unable to write ",r,d,fn,": ");
}

void finish(void)
{
  if (buffer_flush(&obuf) == -1)
    strerr_die6sys(111,FATAL,"unable to write ",r,d,fn,": ");
  close(obuf.fd);
}

void move_file(const char *from,const char *to)
{
  if (rename(from,to) == -1)
    strerr_die6sys(111,FATAL,"unable to rename ",from," to ",to,": ");
}

