#include <unistd.h>
#include <sys/types.h>
#include "auto_scanq.h"
#include "strerr.h"
#include "auto_uids.h"
#include "auto_gids.h"
#include "buffer.h"
#include "auto_cleaner.h"
#include "auto_pkg.h"
#include "generic-install.h"
#include "fifo.h"
#include "open.h"

const char FATAL[] = "install-cleanq: fatal: ";

extern char auto_cqpause[];
extern char auto_cqlog[];

void warn(const char *s)
{
  if (buffer_puts(buffer_1,s) == -1)
    strerr_die2sys(111,FATAL,"unable to write stdout: ");
}

int main(void)
{
  int fdpipe=-1;

  /* Create directories */
  r = (char *)auto_scanq;
  makedir(r,03755,0,0);
  if (chdir(r) == -1)
    strerr_die4sys(111,FATAL,"unable to chdir to ", r, "/log: ");
  makedir("root",02755,0,0);
  makedir("log",02755,0,0);
  makedir("supervise",02770,0,auto_ctlgid);

  /* Touch the "down" file; we only run as needed. */
  fn = "down";start();finish();attrs(fn,0444,0,0);

  /* Create run script */
  fn = "run";
  d  = "/";
  if (!file_exists()) {
    fn = "run.tmp";
    start();
    outs("#!/bin/sh\nexec 2>&1\ncd root/scanq || exit -1\n");
    outs("sleep ");
    outs(auto_cqpause);
    outs("\nexec setuidgid ");
    outs(auto_user);
    outs(" ");
    outs(auto_cleaner);
    outs(" -l\n");
    finish();
    move_file(fn,"run");
    fn = "run";
    attrs(fn,0744,0,0);
  }
  else {
    warn("WARNING: keeping old run script. If this is wrong, delete file and reinstall\n");
  }

  /* Supervise control dir */
  d = "/supervise/";
  fn = "supervise/lock"; start(); finish();attrs(fn,0660,0,auto_ctlgid);
  fn = "supervise/status";start();finish();attrs(fn,0664,0,auto_ctlgid);
  fn="supervise/control";fifo_make(fn,0660);attrs(fn,0660,0,auto_ctlgid);
  fdpipe = open_read(fn);close(fdpipe);
  if (fdpipe == -1)
    strerr_die4sys(111,FATAL,"unable to read ",fn,": ");
  fn="supervise/ok";fifo_make(fn,0660);attrs(fn,0660,0,auto_ctlgid);
  fdpipe = open_read(fn);close(fdpipe);
  if (fdpipe == -1)
    strerr_die4sys(111,FATAL,"unable to read ",fn,": ");

  /* log dir */
  d  = "/log/";
  if (chdir("log") == -1)
    strerr_die4sys(111,FATAL,"unable to chdir to ", r, "/log: ");
  makedir("main",02755,auto_loguid,0);

  /* Create the "run" script */
  fn = "run";
  if (!file_exists()) {
    fn = "run.tmp";
    start();
    outs("#!/bin/sh\nexec setuidgid ");
    outs(auto_loguser);
    outs(" multilog t \\\n  ");
    outs(auto_cqlog);
    outs("\\\n  ./main\n");
    finish();
    move_file(fn,"run");
    fn = "run";
    attrs(fn,0744,0,0);
  }
  else {
    warn("WARNING: keeping old log/run script. If this is wrong, delete file and reinstall\n");
  }

  /* Touch the "status" file for the logger. On upgrade, might be in use! */
  fn = "status";
  if (!file_exists()) {
    start();
    finish();
    attrs(fn,0644,auto_loguid,0);
  }

  /* root dir */
  if (chdir("../root") == -1)
    strerr_die4sys(111,FATAL,"unable to chdir to ", r, "/root: ");
  makedir("scanq",02700,auto_uid,0);

  /* crontab */
  fn = "crontab";
  d  = "/root/";
  if (!file_exists()) {
    fn = "crontab.tmp";
    start();
    outs("12 * * * *	");
    outs(auto_pkg);
    outs("/command/run-cleanq\n");
    finish();
    move_file(fn,"crontab");
    fn = "crontab";
    attrs(fn,0644,0,0);
  }
  else {
    warn("WARNING: keeping old crontab. If this is wrong, delete crontab and reinstall\n");
  }

  return(0);
}
