#include <unistd.h>
#include <sys/types.h>
#include <stdio.h> /* Needed for rename() */
#include "auto_scanq.h"
#include "str.h"
#include "strerr.h"
#include "auto_gids.h"
#include "buffer.h"
#include "auto_pkg.h"
#include "generic-install.h"

const char *FATAL="install-wrap: fatal: ";

extern const char auto_qmailq_destdir[];
extern const char auto_qmailq_oldpath[];
extern const char auto_qmailbin[];
extern const char auto_qscanq[];

int main(void)
{
  /* Sanity check */
  if (str_diff(auto_qmailbin,auto_qmailq_destdir) == 0) {
    buffer_puts(buffer_2,FATAL);
    buffer_putsflush(buffer_2,"qmail/bin and qmailq_destdir are the same.\n");
    _exit(111);
  }

  /* Create directories */
  makedir(auto_qmailq_destdir,0755,0,0);
  if (chdir(auto_qmailq_destdir) == -1)
    strerr_die4sys(111,FATAL,"unable to chdir to ", auto_qmailq_destdir,": ");
  fn = "qmail-queue";
  if (!file_exists())
    if (link(auto_qmailq_oldpath,"qmail-queue")==-1)
      strerr_die6sys(111,FATAL,"unable to link ", auto_qmailq_oldpath, " to ",auto_qmailq_destdir, "/qmail-queue: ");
  if (chdir(auto_qmailbin) == -1)
    strerr_die4sys(111,FATAL,"unable to chdir to ", auto_qmailbin,": ");
  if (symlink(auto_qscanq, "qmail-queue.tmp") == -1)
    strerr_die6sys(111,FATAL,"unable to symlink ", auto_qscanq, " to ",auto_qmailbin, "/qmail-queue.tmp: ");
  if (rename("qmail-queue.tmp","qmail-queue") == -1)
    strerr_die2sys(111,FATAL,"unable to rename qmail-queue.tmp to qmail-queue: ");
  return(0);
}
