#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "exit.h"
#include "auto_spool.h"
#include "auto_fnlen.h"
#include "mkfn.h"
#include "auto_retries.h"
#include "fork.h"
#include "wait.h"
#include "sig.h"
#include "exitcodes.h"
#include "buffer.h"
#include "strerr.h"
#include "env.h"

int do_cleanq();

extern char *const auto_qstdin[];
int flaglog = 0;

#define FATAL "qscanq: fatal: "
void err(const char *s)
{
  if (flaglog && (buffer_putsflush(buffer_2,s) == -1))
    strerr_die2sys(QQ_XTEMP,FATAL,"unable to write stderr: ");
}

/* Global so cleanup routine can find it. */
char fn[FN_BYTES];
void cleanup()
{
  unlink("msg");	/* just in case */
  chdir(auto_spool);
  chmod(fn, 0700);
  rmdir(fn);	/* if it fails, bummer */
}
void real_cleanup()
{
  cleanup();
  do_cleanq();
}

int
main (int argc, char *argv[])
{
  int c=1;
  pid_t pid = 0;
  int qstat = -1;
  int opt;

  /* Check whether we should be logging errors */
  if (env_get("DEBUG")) flaglog = 1;

  /* Just in case! */
  umask(0);

  /* [ cwd := spool folder ] */
  if (chdir(auto_spool) < 0) {
    if (flaglog)
      strerr_die4sys(QQ_XTEMP,FATAL,"unable to chdir to ",auto_spool,": ");
    _exit(QQ_XTEMP); /* temporary refusal to handle messages */
  }

  /* [fn := name timestamp.ppid.n of existing directory w/sticky bit set ] */
  do {
    mkfn(fn, c++);
    if (mkdir(fn, 0700) == 0)
      if (chmod(fn, 01700) == 0) break;
    sleep(1);
  } while (c<MAX_RETRIES);
  if (c == MAX_RETRIES) {
    if (flaglog)
      strerr_die4sys(QQ_XTEMP,FATAL,"unable to mkdir ",fn,": ");
    _exit(QQ_XTEMP);
  }

  /* [ cwd := fn ] */
  if (chdir(fn) < 0) {
    if (flaglog)
      strerr_die4sys(QQ_XTEMP,FATAL,"unable to chdir to ",fn,": ");
    rmdir(fn); /*if rmdir fails, bummer.*/
    _exit(QQ_XTEMP);
  }

  /* [ cwd := fn/work */
  if (mkdir("work",0777) == -1) {
    if (flaglog)
      strerr_die4sys(QQ_XTEMP,FATAL,"unable to mkdir ",fn,"/work: ");
    cleanup();
    _exit(QQ_XTEMP);
  }
  if (chdir("work") == -1) {
    if (flaglog)
      strerr_die4sys(QQ_XTEMP,FATAL,"unable to chdir to ",fn,"/work: ");
    real_cleanup();
    _exit(QQ_XTEMP);
  }

  /* Run a non-privileged process to do actual scanning */
  switch(pid = vfork()) {
    case -1:
	return 0;
    case 0:
	setreuid(getuid(),getuid());
	if (argc>1) execv(auto_qstdin[0], argv);
	else execv(*auto_qstdin, auto_qstdin);
        if (flaglog) strerr_die2sys(QQ_XTEMP,FATAL,"execv failed: ");
	real_cleanup();
	_exit(QQ_XTEMP); /* hopefully never reached */;
  }

  /* Catch the exit status */
  if (wait_pid(&qstat, pid) == -1) {
    if (flaglog) strerr_die2sys(QQ_XTEMP,FATAL,"waidpid failed: ");
    real_cleanup();
    _exit(QQ_XTEMP); 
  }
  real_cleanup();
  if (wait_crashed(qstat)) {
    if (flaglog) err("qscanq-stdin crashed.\n");
    _exit(QQ_XTEMP);
  }
  _exit(wait_exitcode(qstat));
}
